@@ -562,6 +562,94 @@ class CatalogHackTests(utils.TestCase):
 
         self.assertEqual(self.V3_URL, endpoint)
 
+    def test_setting_no_discover_hack(self):
+        v2_disc = fixture.V2Discovery(self.V2_URL)
+        common_disc = fixture.DiscoveryList(href=self.BASE_URL)
+
+        v2_m = self.stub_url('GET',
+                             ['v2.0'],
+                             base_url=self.BASE_URL,
+                             status_code=200,
+                             json=v2_disc)
+
+        common_m = self.stub_url('GET',
+                                 [],
+                                 base_url=self.BASE_URL,
+                                 status_code=300,
+                                 json=common_disc)
+
+        resp_text = uuid.uuid4().hex
+
+        resp_m = self.stub_url('GET',
+                               ['v3', 'path'],
+                               base_url=self.BASE_URL,
+                               status_code=200,
+                               text=resp_text)
+
+        # it doesn't matter that we auth with v2 here, discovery hack is in
+        # base. All identity endpoints point to v2 urls.
+        token = fixture.V2Token()
+        service = token.add_service(self.IDENTITY)
+        service.add_endpoint(public=self.V2_URL,
+                             admin=self.V2_URL,
+                             internal=self.V2_URL)
+
+        self.stub_url('POST',
+                      ['tokens'],
+                      base_url=self.V2_URL,
+                      json=token)
+
+        v2_auth = identity.V2Password(self.V2_URL,
+                                      username=uuid.uuid4().hex,
+                                      password=uuid.uuid4().hex)
+
+        sess = session.Session(auth=v2_auth)
+
+        # v2 auth with v2 url doesn't make any discovery calls.
+        self.assertFalse(v2_m.called)
+        self.assertFalse(common_m.called)
+
+        # v3 endpoint with hack will strip v2 suffix and call root discovery
+        endpoint = sess.get_endpoint(service_type=self.IDENTITY,
+                                     version=(3, 0),
+                                     allow_version_hack=True)
+
+        # got v3 url
+        self.assertEqual(self.V3_URL, endpoint)
+
+        # only called root discovery.
+        self.assertFalse(v2_m.called)
+        self.assertTrue(common_m.called_once)
+
+        # with hack turned off it calls v2 discovery and finds nothing
+        endpoint = sess.get_endpoint(service_type=self.IDENTITY,
+                                     version=(3, 0),
+                                     allow_version_hack=False)
+        self.assertIsNone(endpoint)
+
+        # this one called v2
+        self.assertTrue(v2_m.called_once)
+        self.assertTrue(common_m.called_once)
+
+        # get_endpoint returning None raises EndpointNotFound when requesting
+        self.assertRaises(exceptions.EndpointNotFound,
+                          sess.get,
+                          '/path',
+                          endpoint_filter={'service_type': 'identity',
+                                           'version': (3, 0),
+                                           'allow_version_hack': False})
+
+        self.assertFalse(resp_m.called)
+
+        # works when allow_version_hack is set
+        resp = sess.get('/path',
+                        endpoint_filter={'service_type': 'identity',
+                                         'version': (3, 0),
+                                         'allow_version_hack': True})
+
+        self.assertTrue(resp_m.called_once)
+        self.assertEqual(resp_text, resp.text)
+
 
 class GenericPlugin(plugin.BaseAuthPlugin):
 
@@ -613,7 +701,7 @@ class GenericAuthPluginTests(utils.TestCase):
 
         self.assertEqual(text, resp.text)
 
-        for k, v in six.iteritems(self.auth.headers):
+        for k, v in self.auth.headers.items():
             self.assertRequestHeaderEqual(k, v)
 
         self.assertIsNone(self.session.get_token())
